<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Console\Command;

use Exception;
use FiloBlu\MappConnector\Helper\FileLoggerHelper;
use FiloBlu\MappConnector\Model\Config;
use FiloBlu\MappConnector\Model\Jobs;
use FiloBlu\MappConnector\Model\Mapp;
use FiloBlu\MappConnector\Model\MappCSV;
use Magento\Framework\App\Area;
use Magento\Framework\App\State;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Prepare
 * @package FiloBlu\MappConnector\Console\Command
 */
class Prepare extends Command
{

    /**
     * @var Jobs
     */
    protected $_jobs;

    /**
     * @var Mapp
     */
    protected $_mapp;

    /**
     * @var MappCSV
     */
    protected $_mappCSV;

    /**
     * @var FileLoggerHelper
     */
    protected $_loggerHelper;

    /**
     * @var State
     */
    protected $_state;

    /**
     * @var Config
     */
    protected $_config;

    /**
     * @var array
     */
    protected $_entities = ['customer', 'subscriber', 'order', 'product', 'quote'];

    /**
     * Prepare constructor.
     * @param Mapp $mapp
     * @param MappCSV $mappCSV
     * @param Jobs $jobs
     * @param State $state
     * @param FileLoggerHelper $loggerHelper
     * @param Config $config
     * @param string|null $name
     */
    public function __construct(
        Mapp $mapp,
        MappCSV $mappCSV,
        Jobs $jobs,
        State $state,
        FileLoggerHelper $loggerHelper,
        Config $config,
        string $name = null
    ) {
        $this->_jobs = $jobs;
        $this->_mapp = $mapp;
        $this->_mappCSV = $mappCSV;
        $this->_state = $state;
        $this->_loggerHelper = $loggerHelper;
        $this->_config = $config;
        parent::__construct($name);
    }

    /**
     * {@inheritdoc}
     * @throws Exception
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        //Area Code Fix
        try {
            $this->_state->setAreaCode(Area::AREA_GLOBAL);
        } catch (Exception $e) {
            //Area code is already set
        }

        $entity = strtolower(trim($input->getArgument('entity')));
        $t = strtolower(trim($input->getArgument('type')));
        $m = strtolower(trim($input->getArgument('mode')));

        //Prepare Jobs
        $this->prepareJobs($entity, $t, $m, $output);
    }

    /**
     * @param null $entity
     * @param null $t
     * @param null $m
     * @param OutputInterface|null $output
     * @throws Exception
     */
    public function prepareJobs($entity = null, $t = null, $m = null, OutputInterface $output = null)
    {
        $type = 'incremental';
        if (!empty($t) && in_array($t, ['incremental', 'full', 'fullincremental'])) {
            $type = $t;
        }

        $mode = 'sftp';
        if (!empty($m) && in_array($m, ['sftp', 'rest'])) {
            $mode = $m;
            if ($mode == 'rest' && !in_array($entity, ['customer', 'subscriber'])) {
                if ($output) {
                    $output->writeln("\n-- WARNING: rest type is allowed only on customer and subscriber --\n");
                }
                return;
            }
        }

        //If entity is specified and exist process entity
        if (!empty($entity) && $entity != 'all') {
            if (in_array($entity, $this->_entities)) {
                $this->_mappCSV->setJobType($entity);
                $this->_mappCSV->prepareExport($type, $mode);
            } else {
                if ($output) {
                    $output->writeln("WARNING: The entity specified {$entity} doesn't exists");
                }
            }
        } // Else process all entities
        else {
            foreach ($this->_entities as $entity_item) {
                $this->_mappCSV->setJobType($entity_item);
                $this->_mappCSV->prepareExport($type, $mode);
            }
        }

        // To backup rest jobs on ftp
        $this->_convertCompletedRestToFtpJobs();
    }

    protected function _convertCompletedRestToFtpJobs()
    {
        if ($this->_config->getConfigValue('mappconnector/general/enable_ftp_backup_rest') == 1) {
            $ids = $this->_jobs->getAllCompletedRestIds();
            $this->_jobs->massUpdateJob($ids, 'waiting', 'sftp');

            $this->_loggerHelper->debug('Converted to SFTP ' . count($ids) . ' Rest processes');
        }
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName('mapp:prepare');
        $this->setDescription('Prepare full export');
        // Next line will add a new required parameter to our script
        $this->addArgument('entity', InputArgument::OPTIONAL, __('Please specify an Entity'));
        $this->addArgument('type', InputArgument::OPTIONAL, __('Please specify type of export'));
        $this->addArgument('mode', InputArgument::OPTIONAL, __('Please specify mode of export'));
        parent::configure();
    }
}
