<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

use Magento\Framework\App\ResourceConnection;

/**
 * Class AbstractEntity
 * @package FiloBlu\MappConnector\Model\Entity
 */
abstract class AbstractEntity implements EntityInterface
{
    /**
     * @var
     */
    protected $_model;

    /**
     * @var string
     */
    protected $_entity;

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel;

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary;

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary;

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource;

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary;

    /**
     * User Type
     *
     * @var string
     */
    protected $_userType;

    /**
     * @var string
     */
    protected $_xmlConfigPath;

    /** @var */
    protected $_config;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * Quote constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(ResourceConnection $resourceConnection)
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return string
     */
    public function getEntity()
    {
        return $this->_entity;
    }

    /**
     * @return string
     */
    public function getEntityModel()
    {
        return $this->_entityModel;
    }

    /**
     * @return string
     */
    public function getEntityModelPrimary()
    {
        return $this->_entityModelPrimary;
    }

    /**
     * @return string
     */
    public function getNormalizedEntityModelPrimary()
    {
        return $this->_entityModelPrimary;
    }

    /**
     * @return string
     */
    public function getEntityModelRelatedPrimary()
    {
        return $this->_entityModelRelatedPrimary;
    }

    /**
     * @return string
     */
    public function getEntityMappResource()
    {
        return $this->_entityMappResource;
    }

    /**
     * @return array
     */
    public function getEntityMappPrimary()
    {
        return $this->_entityMappPrimary;
    }

    /**
     * @return string
     */
    public function getUserType()
    {
        return $this->_userType;
    }

    /**
     * @return string
     */
    public function getXmlConfigPath()
    {
        return $this->_xmlConfigPath;
    }

    /**
     * @param $record
     * @return array
     */
    public function getDefaultData($record)
    {
        return [
            $this->_entityMappPrimary['primaryMapp'] => $record[$this->_entityMappPrimary['primaryDB']]
        ];
    }

    /**
     * @param $primary
     * @param $id
     * @return array
     */
    public function getObjectData($primary, $id)
    {
        return [];
    }

    /**
     * @return array
     */
    public function getAllData()
    {
        return [];
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record)
    {
        return [];
    }

    /**
     * @return \Magento\Sales\Model\Order
     */
    public function getModel()
    {
        return $this->_model;
    }

    /**
     * @return bool
     */
    public function canProcess()
    {
        return true;
    }

    /**
     * @param $relatedModel
     * @param $id
     * @return mixed
     */
    protected function getRelatedModelData($relatedModel, $id)
    {
        $map = $this->_config->getNormalizedConfigObjectByPath($relatedModel->getXmlConfigPath(), $relatedModel->getEntity());
        $relatedData = $relatedModel->getObjectData($relatedModel->getEntityModelRelatedPrimary(), $id);

        //If not exists related record return an empty filled array
        if (!count($relatedData)) {
            return array_fill_keys($map, null);
        } else {
            return $this->_config->getMappedArray($map, $relatedData);
        }
    }
}
